unit SearchTextFrm;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, HexEdit;

type
  TSearchTextData = record
    Text: string;
    AllDoc: Boolean;
    Opt: TSearchTextOptions;
  end;

  TSearchTextForm = class(TForm)
    Label1: TLabel;
    TextComboBox: TComboBox;
    GroupBox1: TGroupBox;
    UseWildCheckBox: TCheckBox;
    Label2: TLabel;
    WildCharEdit: TEdit;
    Label3: TLabel;
    WildCountEdit: TEdit;
    GroupBox2: TGroupBox;
    MatchCaseCheckBox: TCheckBox;
    WholeWordCheckBox: TCheckBox;
    OnlyBlockCheckBox: TCheckBox;
    AllDocCheckBox: TCheckBox;
    UnicodeCheckBox: TCheckBox;
    SearchButton: TButton;
    CancelButton: TButton;
    HelpButton: TButton;
    DefaultButton: TButton;
    procedure SearchButtonClick(Sender: TObject);
    procedure CancelButtonClick(Sender: TObject);
    procedure HelpButtonClick(Sender: TObject);
    procedure DefaultButtonClick(Sender: TObject);
    procedure FormActivate(Sender: TObject);
    procedure UseWildCheckBoxClick(Sender: TObject);
    procedure FormCreate(Sender: TObject);
  private
    { Private declarations }
    function CheckValid: Boolean;
  public
    { Public declarations }
    procedure GetData(var Data: TSearchTextData);
  end;

var
  SearchTextForm: TSearchTextForm;

function ShowSearchTextForm(var Data: TSearchTextData): Boolean;

implementation

uses Misc, NumBase;

{$R *.DFM}

function ShowSearchTextForm(var Data: TSearchTextData): Boolean;
begin
  Result := (SearchTextForm.ShowModal = mrOk);
  if Result then SearchTextForm.GetData(Data);
end;

procedure TSearchTextForm.GetData(var Data: TSearchTextData);
begin
  Data.Text := TextComboBox.Text;
  Data.AllDoc := AllDocCheckBox.Checked;
  Data.Opt.MatchCase := MatchCaseCheckBox.Checked;
  Data.Opt.WholeWord := WholeWordCheckBox.Checked;
  Data.Opt.Unicode := UnicodeCheckBox.Checked;
  Data.Opt.OnlyBlock := OnlyBlockCheckBox.Checked;
  Data.Opt.UseWild := UseWildCheckBox.Checked;
  Data.Opt.WildChar := WildCharEdit.Text[1];
  Data.Opt.WildCount := StrToIntDef(WildCountEdit.Text, 1);
end;

function TSearchTextForm.CheckValid: Boolean;
begin
  Result := True;
  if TextComboBox.Text = '' then
  begin
    Result := False;
    TextComboBox.SetFocus;
    Exit;
  end;
  if Length(WildCharEdit.Text) <> 1 then
  begin
    Result := False;
    MsgBox('ͨȷ');
    WildCharEdit.SetFocus;
    Exit;
  end;
  if not IsDecStr(WildCountEdit.Text) then
  begin
    Result := False;
    MsgBox('ֽͨȷ');
    WildCountEdit.SetFocus;
    Exit;
  end;
end;

procedure TSearchTextForm.FormCreate(Sender: TObject);
begin
  UseWildCheckBoxClick(nil);
end;

procedure TSearchTextForm.SearchButtonClick(Sender: TObject);
begin
  if CheckValid then
  begin
    TextComboBox.Items.Insert(0, TextComboBox.Text);
    ModalResult := mrOk;
  end;
end;

procedure TSearchTextForm.CancelButtonClick(Sender: TObject);
begin
  ModalResult := mrCancel;
end;

procedure TSearchTextForm.HelpButtonClick(Sender: TObject);
begin
//
end;

procedure TSearchTextForm.DefaultButtonClick(Sender: TObject);
begin
  TextComboBox.Text := '';
  MatchCaseCheckBox.Checked := False;
  WholeWordCheckBox.Checked := False;
  UnicodeCheckBox.Checked := False;
  OnlyBlockCheckBox.Checked := False;
  AllDocCheckBox.Checked := False;
  UseWildCheckBox.Checked := False;
  WildCharEdit.Text := '?';
  WildCountEdit.Text := '1';
end;

procedure TSearchTextForm.FormActivate(Sender: TObject);
begin
  TextComboBox.SetFocus;
end;

procedure TSearchTextForm.UseWildCheckBoxClick(Sender: TObject);
begin
  if UseWildCheckBox.Checked then
  begin
    WildCharEdit.Enabled := True;
    WildCountEdit.Enabled := True;
    WildCharEdit.Color := clWindow;
    WildCountEdit.Color := clWindow;
  end else
  begin
    WildCharEdit.Enabled := False;
    WildCountEdit.Enabled := False;
    WildCharEdit.Color := clBtnFace;
    WildCountEdit.Color := clBtnFace;
  end;
end;

end.
